/*

 reads values of analog inputs (0-5) of arduino board
 and vizualizes them as a graph
 
 */

// libraries
import processing.serial.*;
import processing.opengl.*;
// serial connection
Serial port;
String message = null;
String elements[] = null;
// arduino
int[] sensorValues = new int[6];
int[] difSensorValues = new int[6];
int[] oldSensorValues = new int[6];
// pong
PFont fontGraph;
// players
int barWidth = 200;
int barHeight = 20;
int[] posX = new int[4];
int[] posY = new int[4];
// ball
int sizeBall = 20;
float posXBall = 320.0;
float posYBall = 320.0;
float speedX = 2.0;
float speedY = 0.0;
float random1 = 0.6;
float random2 = 1.5;

void setup() {

  // size
  size(640, 640, OPENGL);
  // no strokes
  noStroke();
  // color
  fill(255);
  // Set the font and its size (in units of pixels)
  fontGraph = loadFont("Verdana.vlw");
  textFont(fontGraph, 64);
  // print all available serial ports
  println(Serial.list());
  // init connection to arduino
  port = new Serial(this, Serial.list()[0], 57600);

}

void draw() {

  // clear background
  background(0);

  /////////////
  // players //
  /////////////

  // player one
  posX[0] = 0;
  posY[0] = (int)map(sensorValues[0], 0, 1023, 0, height-barWidth);
  rect(posX[0],posY[0],barHeight,barWidth);

  // player two
  posX[1] = (int)map(sensorValues[1], 0, 1023, 0, width-barWidth);
  posY[1] = 0;
  rect(posX[1],posY[1],barWidth,barHeight);

  // player three
  posX[2] = width-barHeight;
  posY[2] = (int)map(sensorValues[2], 0, 1023, 0, width-barWidth);
  rect(posX[2],posY[2],barHeight,barWidth);

  // player four
  posX[3] = (int)map(sensorValues[3], 0, 1023, 0, width-barWidth);
  posY[3] = height-barHeight;
  rect(posX[3],posY[3],barWidth,barHeight);


  //////////
  // ball //
  //////////

  // position
  posXBall = posXBall + speedX;
  posYBall = posYBall + speedY;

  // check player 1
  if((posXBall <= posX[0] + barHeight)&&(posXBall > 0)){
    if((posYBall >= posY[0])&&(posYBall <= posY[0]+barWidth)){
      // change X speed
      speedX = speedX*(-1.0);
      // change Y speed
      speedY = speedY+difSensorValues[0]/10.0;
    } 

  } 
  // check player 2
  if((posXBall >= posX[1])&&(posXBall <= posX[1]+barWidth)){
    if((posYBall > 0)&&(posYBall <= barHeight)){
      // change X speed
      speedX = speedX+difSensorValues[1]/10.0;
      // change Y speed
      speedY = speedY*(-1.0);
    }
  }


  // check player 3
  if((posXBall >= posX[2])&&(posXBall < width)){
    if((posYBall >= posY[2])&&(posYBall <= posY[2]+barWidth)){
      // change X speed
      speedX = speedX*(-1.0);
      // change Y speed
      speedY = speedY+difSensorValues[2]/10.0;
    }
  }

  // check player 4
  if((posXBall >= posX[3])&&(posXBall <= posX[3]+barWidth)){
    if((posYBall < height)&&(posYBall >= height-barHeight)){
      // change X speed
      speedX = speedX+difSensorValues[3]/10.0;
      // change Y speed
      speedY = speedY*(-1.0);
    }
  }

  // reset
  if((posXBall < 0 - barHeight)||(posXBall > width + barHeight)||(posYBall < 0 - barHeight)||(posYBall > height + barHeight)){
    reset();
  }

  // draw ball
  ellipseMode(CENTER);
  ellipse(posXBall,posYBall,sizeBall,sizeBall);



}

void reset(){
  //
  delay(1000);
  // reset
  posXBall = 320.0;
  posYBall = 320.0;
  speedX = 2.0;
  speedY = 0.0;
}

// serial event
void serialEvent(Serial p){

  // get message till line break (ASCII > 13) 
  message = port.readStringUntil(13);
  // just if there is data
  if (message != null) {
    // try catch function because of possible garbage in received data
    try {
      elements = splitTokens(message);
      // loop through sensor values
      for (int i =0; i < elements.length && i < 6; i++ ) {
        sensorValues[i] = int(elements[i]);
        difSensorValues[i] = sensorValues[i] - oldSensorValues[i];
        oldSensorValues[i] = sensorValues[i];
        print(difSensorValues[i]);
        print("_");
      }
      println("");
    }
    catch (Exception e){
    }
  }

}

















